"use strict";

this.name = "yah_gem_holdem";
this.author = "spara";

this.$highNames = ["Seven", "Eight", "Nine", "Ten", "Jack", "Queen", "King", "Ace"];
this.$handRanks = ["High", "Pair","Two Pair","Three of a Kind","Straight","Flush","Full House","Four of a Kind","Straight Flush","Royal Flush"];

//add interface, if the player has started the game anywhere elsewhere than constore
this.startUpComplete = function() {
	if (!player.ship.dockedStation.hasRole("constore")) {
		this.shipExitedWitchspace();
	}
}

this.shipExitedWitchspace = function() {
	var constores = system.shipsWithRole("constore");
	if (constores.length > 0) {
		for (var i = 0; i < constores.length; i++) {
			constores[i].setInterface("yah_gem-holdem",{
				title: "hOopy hold'em",
				category: expandMissionText("yah_gem-gambling-category"),
				summary: expandMissionText("yah_gem-holdem-summary"),
				callback: this.$intro.bind(this)
			});
		}
	}
}

//init a new deck
this.$virginDeck = function() {
	var deck = new Array();
	var suits = ["triangle", "crescent", "square", "star"];
	for (var i = 0; i < 4; i++) {
		for (var j = 1; j < 14; j++) {
			var card = new Object();
			card.suit = suits[i];
			card.rank = j;
			card.show = false;
			deck.push(card);
		}
	}
	return deck;
}

//draws a card from the current deck in play
this.$drawCard = function() {
	return this.$playDeck.splice(Math.floor(this.$playDeck.length * Math.random()),1)[0];
}

this.$intro = function() {
	var credits = "\n\nGems: " + player.ship.manifest["gem_stones"] + " g";
	var options = {};
	options["1_EXIT"] = "Exit";
	if (player.ship.manifest["gem_stones"] >= 3)
		options["2_10"] = "Bet 1";
	else credits += expandMissionText("yah_gem-no-credit") + " Minimum of 3 g is needed to play.";
	if (player.ship.manifest["gem_stones"] >= 15)
		options["3_50"] = "Bet 5";
	if (player.ship.manifest["gem_stones"] >= 30)
		options["4_100"] = "Bet 10";
	
	mission.runScreen({
		titleKey: "yah_gem-holdem-title",
		message: expandMissionText("yah_gem-holdem-intro-story") + credits,
		model: "yah_gem_casino_holdem_intro",
		background: "yah_gem_table_bg.png",
		choices: options,
		allowInterrupt: true,
		screenID: "yah_gem_holdem",
		exitScreen: "GUI_SCREEN_INTERFACES"
	},
	function (choice) {
		if (choice === "2_10") this.$play(1);
		else if (choice === "3_50") this.$play(5);
		else if (choice === "4_100") this.$play(10);
	}	
	);
}

//first pay, then deal
this.$play = function(bet) {
	player.ship.manifest["gem_stones"] -= bet;
	this.$ante = bet;
	this.$bets = 0;
	this.$playDeck = this.$virginDeck();
	this.$cards = new Array();
	for (var i = 0; i < 9; i++) this.$cards.push(this.$drawCard());
	
	//debug hands
	//var deal = [48, 41, 5, 6, 23, 33, 19, 21, 30];
	//for (var i = 0; i < 9; i++) {
	//	this.$cards.push(this.$playDeck[deal[i]]);
	//}
	
	this.$deal("deal");
}

this.$deal = function(phase) {
	var message = "Ante: " + this.$ante;
	if (this.$bets > 0) message += "\nBets: " + this.$bets;
	message += "\n\nGems: " + player.ship.manifest["gem_stones"] + " g";
	var options = {};
	switch(phase) {
		case "deal" :	this.$cards[7].show = true; 
						this.$cards[8].show = true;
						options["1_FLOP"] = "Play (" + this.$ante * 2 + " g)";
						options["2_FOLD"] = "Fold";
						break;
		case "flop" :	this.$cards[2].show = true; 
						this.$cards[3].show = true;
						this.$cards[4].show = true;
						options["3_CHECK_TURN"] = "Check";
						if (player.ship.manifest["gem_stones"] >= this.$ante)
							options["4_BET_TURN"] = "Bet (" + this.$ante + " g)";
						break;
		case "turn" :	this.$cards[5].show = true; 
						options["5_CHECK_RIVER"] = "Check";
						if (player.ship.manifest["gem_stones"] >= this.$ante)
							options["6_BET_RIVER"] = "Bet (" + this.$ante + " g)";
						break;
	};
	mission.runScreen({
		titleKey: "yah_gem-holdem-title",
		model: "yah_gem_casino_holdem_base",
		message: message,
		background: "yah_gem_table_bg.png",
		spinModel: false,
		screenID: "yah_gem_holdem",
		choices: options
	},
	function(choice) {
		switch(choice) {
			case "1_FLOP" : player.ship.manifest["gem_stones"] -= 2 * this.$ante;
							this.$bets += 2 * this.$ante;
							this.$deal("flop");
							return;
			case "2_FOLD" : this.$showDown(true);
							return;
			case "3_CHECK_TURN" : 	this.$deal("turn");
									return;
			case "4_BET_TURN" : player.ship.manifest["gem_stones"] -= this.$ante;
								this.$bets += this.$ante;
								this.$deal("turn");
								return;
			case "5_CHECK_RIVER" : 	this.$showDown(false);
									return;
			case "6_BET_RIVER" : 	player.ship.manifest["gem_stones"] -= this.$ante;
									this.$bets += this.$ante;
									this.$showDown(false);
									return;
		}
	});
	this.$updateCards();
}

this.$handName = function(value, high) {
	if (value > 0)
		return this.$handRanks[value];
	else
		return this.$highNames[high - 7] + " " + this.$handRanks[0];
}

this.$formatHand = function(hand) {
	var formatted = "(";
	for (var i = 0; i < 5; i++) {
		var value = hand[0][i].value;
		if (value === 11) value = "J";
		else if (value === 12) value = "Q";
		else if (value === 13) value = "K";
		else if (value === 14 || value === 1) value = "A";
		formatted += value;
		if (i < 4) formatted += ", ";
		else formatted += ")";
	}
	return formatted;
}

this.$showDown = function(fold) {
	for (var i = 0; i < 9; i++) this.$cards[i].show = true;
	var playerHand = this.$bestHand(2);
	var dealerHand = this.$bestHand(0);
	var playerValues = "\n" + this.$formatHand(playerHand) + "\n\n";
	var dealerValues = "\n" + this.$formatHand(dealerHand) + "\n\n";
	var message = "Player: " + this.$handName(playerHand[1], playerHand[0][0].value) + playerValues;
	message += "Dealer: " + this.$handName(dealerHand[1], dealerHand[0][0].value) + dealerValues;
	
	if (fold) var result = "fold";
	else {
		var result = "push";
		if (dealerHand[1] > playerHand[1])
			result = "loss";
		else if (dealerHand[1] < playerHand[1]) {
			result = "win";
		}
		else {
			for (var i = 0; i < 5; i++) {
				if (playerHand[0][i].value > dealerHand[0][i].value) {
					result = "win";
					break;
				}
				else if (playerHand[0][i].value < dealerHand[0][i].value) {
					result = "loss";
					break;
				}
			}
		}
	}
	var notifySound = new SoundSource;
	switch (result) {
		case "win" : 	if (playerHand[1] > 3) {
							var win = 2 * (this.$bets + this.$ante);
						}
						else var win = this.$ante + 2 * (this.$bets);
						message += "You win " + (win - this.$bets - this.$ante) + " g";
						player.ship.manifest["gem_stones"] += win;
						notifySound.sound = "yah_gem_win.ogg";
						break;
		case "loss" : 	message += "Dealer wins, you lose " + this.$bets + " g";
						notifySound.sound = "yah_gem_lose.ogg";
						break;
		case "push" : 	message += "It's a push, wagers returned.";
						notifySound.sound = "yah_gem_win.ogg";
						player.ship.manifest["gem_stones"] += this.$bets + this.$ante;
						break;
		case "fold" :	message += "You fold and lose " + this.$ante + " g";
						notifySound.sound = "yah_gem_lose.ogg";
						break;
	}
	notifySound.play();
	message += "\n\nGems: " + player.ship.manifest["gem_stones"] + " g";
	var options = {};
	if (player.ship.manifest["gem_stones"] >= 3 * this.$ante)
		options["7_DEAL"] = "Deal again (" + this.$ante + " g)";
	options["8_BET"] = "Change ante";
	options["9_EXIT"] = "Exit";
	mission.runScreen({
		titleKey: "yah_gem-holdem-title",
		model: "yah_gem_casino_holdem_base",
		message: message,
		background: "yah_gem_table_bg.png",
		spinModel: false,
		choices: options,
		allowInterrupt: true,
		screenID: "yah_gem_holdem",
		exitScreen: "GUI_SCREEN_INTERFACES"
	},
	function(choice) {
		switch(choice) {
			case "7_DEAL" : this.$play(this.$ante);
							return;
			case "8_BET" : this.$intro(1);
							return;
		};
	});
	this.$updateCards();
}

this.$updateCards = function() {
	for (var i = 0; i < 9; i++) {
		var card = this.$cards[i];
		if (card.show) {
			var fileName = "yah_gem_card_" + card.suit + "_" + card.rank + ".png";
		}
		else var fileName = "yah_gem_holdem_card_diffuse.png";
		mission.displayModel.subEntities[i].setMaterials({"casino_poker_card_diffuse.png": {diffuse_map: fileName, emission_map: fileName, emission_modulate_color: "darkGrayColor"}});
	}
}

this.$bestHand = function(start) {
	
	//make a new hand with new objects
	var hand = new Array();
	for (var i = start; i < start + 7; i++) {
		var suits = ["triangle", "crescent", "square", "star"];
		var card = new Object();
		card.value = this.$cards[i].rank;
		card.suit = suits.indexOf(this.$cards[i].suit);
		if (card.value === 1) card.value = 14;
		hand.push(card);
	}
	
	//order cards by suit
	hand.sort(function(a, b) {return a.suit - b.suit});

	//handle flushes first	
	if (hand[0].suit === hand[4].suit || hand[1].suit === hand[5].suit || hand[2].suit === hand[6].suit) {
		//suit of flush
		var suit = hand[2].suit;
		//remove wrong suit cards
		for (var i = 0; i < 7; i++) {
			if (hand[6 - i].suit !== suit)
				hand.splice(6 - i, 1);
		}
		//order cards by value
		hand.sort(function(a, b) {return b.value - a.value})
		//handle possible ace by duplicating to a different value
		if (hand[0].value === 14) {
			card = new Object();
			card.value = 1;
			card.suit = suit;
			hand.push(card);
		}
		//look for a straight
		var straight = -1;
		var length = 0;
		for (var i = 0; i < hand.length - 1; i++) {
			//tracking a straight
			if (hand[i].value - hand[i + 1].value === 1) length++;
			else if (hand[i].value - hand[i + 1].value != 0) length = 0;
			if (length === 4) {
				//highest straight starts from index i - 3
				straight = i - 3;
				break;
			}
		}
		//straight flush found
		if (straight > -1) {
			hand = hand.splice(straight, 5);
			//royal flush
			if (hand[0].value === 14)
				return [hand, 9];
			//straight flush
			else return [hand, 8];
		}
		//a plain flush found
		else {
			//take five most valuable cards
			hand = hand.splice(0, 5);
			return [hand, 5];
		}
	}

	//order cards by value
	hand.sort(function(a, b) {return b.value - a.value});
	
	//handle possible ace by duplicating to a different value
	if (hand[0].value === 14) {
		card = new Object();
		card.value = 1;
		card.suit = hand[0].suit;
		hand.push(card);
	}

	//look for a straight
	var straight = -1;
	var length = 0;
	for (var i = 0; i < hand.length - 1; i++) {
		//tracking a straight
		if (hand[i].value - hand[i + 1].value === 1) length++;
		else if (hand[i].value - hand[i + 1].value != 0) length = 0;
		if (length === 4) {
			//highest straight starts from index i - 3
			straight = i - 3;
			break;
		}
	}

	//straight found
	if (straight > -1) {
		hand = hand.splice(straight, 5);
		return [hand, 4];
	}
	
	//remove possible added ace. this was needed for straight search
	if (hand[0].value === 14)
		hand.pop();

	//make a new array of the cards so that cards with the same value are grouped ot sub arrays
	var grouped = new Array();
	grouped.push([hand[0]]);
	for (var i = 1; i < hand.length; i++) {
		//if there is a group for this card value, then push the card into it.
		if (hand[i].value === grouped[grouped.length - 1][0].value)
			grouped[grouped.length - 1].push(hand[i]);
		//otherwise create a new group
		else grouped.push([hand[i]]);
	}

	//sort grouped cards by group size
	grouped.sort(function(a, b) {return b.length - a.length});

	//four of a kind
	if (grouped[0].length === 4) {
		var newHand = grouped[0];
		var kicker = grouped[1][0];
		for (var i = 2; i < grouped.length; i++) {
			if (grouped[i][0] > kicker) kicker = grouped[i][0];
		}
		newHand.push(kicker);
		return [newHand, 7];
	}
	
	//three of a kind or full house
	if (grouped[0].length === 3) {
		var newHand = grouped[0];
		//full house
		if (grouped[1].length > 1) {
			newHand.push(grouped[1][0]);
			newHand.push(grouped[1][1]);
			return [newHand, 6];
		}
		//three of a kind
		else {
			newHand.push(grouped[1][0]);
			newHand.push(grouped[2][0]);
			return [newHand, 3];
		}
	}
	
	//pair or two pair
	if (grouped[0].length === 2) {
		var newHand = grouped[0];
		//two pair
		if (grouped[1].length > 1) {
			newHand.push(grouped[1][0]);
			newHand.push(grouped[1][1]);
			newHand.push(grouped[2][0]);
			return [newHand, 2];
		}
		//pair
		else {
			newHand.push(grouped[1][0]);
			newHand.push(grouped[2][0]);
			newHand.push(grouped[3][0]);
			return [newHand, 1];
		}
	}

	hand = hand.splice(0, 5);
	return [hand, 0];
}
